import { VNodeChildElement } from './renderer';
interface BaseProps {
    defaultSlot?: VNodeChildElement | VNodeChildElement[];
}
declare type Data<T> = {
    [K in keyof T]: T[K];
};
export interface WebCellComponent<P extends BaseProps = {}, S = {}> extends HTMLElement {
    /**
     * Called every time the element is inserted into the DOM
     */
    connectedCallback?(): void;
    /**
     * Called every time the element is removed from the DOM.
     */
    disconnectedCallback?(): void;
    /**
     * Called when an observed attribute has been added, removed, updated, or replaced.
     * Also called for initial values when an element is created by the parser, or upgraded.
     *
     * Note: only attributes listed in static `observedAttributes` property will receive this callback.
     */
    attributeChangedCallback?(name: string, oldValue: string, newValue: string): void;
    /**
     * The custom element has been moved into a new document
     * (e.g. someone called `document.adoptNode(el)`).
     */
    adoptedCallback?(): void;
    update(): void;
    props: Data<P>;
    setProps(data: {
        [key in keyof P]: any;
    }): Promise<any>;
    state: Data<S>;
    setState(data: {
        [key in keyof S]: any;
    }): Promise<void>;
    defaultSlot: VNodeChildElement[];
    render(props: Data<P>, state: Data<S>): VNodeChildElement | VNodeChildElement[];
    /**
     * Called before `state` is updated
     */
    shouldUpdate?(oldState: S, newState: S): boolean;
    /**
     * Called after rendering
     */
    updatedCallback?(): void;
    emit(event: string, detail?: any, options?: EventInit): boolean;
}
export declare function mixin<P = {}, S = {}>(superClass?: {
    new (): HTMLElement;
    prototype: HTMLElement;
}): {
    new (): WebCellComponent<P, S>;
};
export {};
